CREATE OR REPLACE FUNCTION load_aggr_availability() returns void as $$
DECLARE
    date_it date;
    start_ts TIMESTAMP;
    total_rows INTEGER;
    cur_rows INTEGER;
BEGIN

    -- getting initial timestamp
    start_ts = CLOCK_TIMESTAMP();
    total_rows := 0;

    -- remove old data
    TRUNCATE TABLE aggr_availability;

    -- iterate over past months
    FOR date_it IN SELECT (date_trunc('month', generate_series(
                current_date - interval '4 months',
                current_date,
                interval '1 month')) - interval '1 day')::date UNION select current_date LOOP

        INSERT INTO aggr_availability
        SELECT
            date_it,
            f.macaddr,
            f.machine_type,
            f.tl_id,
            f.tl_name,
            c.name,
            c.state,
            c.region,
            CASE WHEN date_it - max_dat_id <= 10 THEN 1 ELSE 0 END as is_green,
            CASE WHEN date_it - max_dat_id > 10 AND date_it - max_dat_id <= 30 THEN 1 ELSE 0 END as is_yellow,
            CASE WHEN date_it - max_dat_id > 30 THEN 1 ELSE 0 END as is_red
        FROM
            (SELECT
                max(dat_id) as max_dat_id,
                f.tl_id,
		t.cit_id,
		t.name as tl_name,
                f.macaddr,
                f.machine_type
            FROM
                fact_contact f
	    INNER JOIN 
		dim_telecenter t 
	    ON 
		t.id = f.tl_id
            WHERE
                dat_id <= date_it
            GROUP BY
                t.cit_id, tl_name, f.tl_id, f.macaddr, f.machine_type) as f
        INNER JOIN
            dim_city c
        ON
            f.cit_id = c.id;

        -- how many rows were inserted?
        GET DIAGNOSTICS cur_rows := ROW_COUNT;
        total_rows := total_rows + cur_rows;

    END LOOP;

    -- logging
    INSERT INTO control (name, start_time, end_time, total)
        VALUES ('loading aggr_availability', start_ts, CLOCK_TIMESTAMP(), total_rows);
END;
$$ language plpgsql;
