var pg = require('pg'),
    fs = require('fs');

var cfg = null;

exports.config = function (c) {
    cfg = c;
};

exports.connect = function (req, res, next) {
    if (cfg == null)
        throw new Error("Database not configured!");

    pg.connect(cfg, function(err, client, done) {
        if (err) {
            console.log(err);
            done();
            res.send(500, {error: 'db_connection_failed'});
            return;
        }

        req.db = {
            client: client,
            done: done,

            query: function (q, params, cb) {
                client.query(q, params, function (err, result) {
                    if (err) {
                        console.log(err);
                        done();
                        res.send(500, {error: 'db_query_failed'});
                        return;
                    }

                    cb(result);
                });
            },

            queryFromFile: function (file, params, cb) {
                fs.readFile(file, 'utf8', function (err, data) {
                    if (err) {
                        console.log(err);
                        done();
                        res.send(500, {error: 'db_query_failed'});
                        return;
                    }

                    if (typeof params === 'undefined')
                        params = [];

                    client.query(data, params, function (err, result) {
                        if (err) {
                            console.log(err);
                            done();
                            res.send(500, {error: 'db_query_failed'});
                            return;
                        }

                        cb(result);
                    });
                });
            },

            copyFrom: function (q, cb) {
                var stream = client.copyFrom(q);

                stream.on('close', function () {
                    cb();
                });

                stream.on('error', function (err) {
                    req.db.done();
                    console.log(err);
                    res.send(500, {error: 'db_query_failed'});
                });

                return stream;
            },

            transaction: function (cb) {
                client.query('BEGIN;', function (err, result) {
                    if (err) {
                        console.log(err);
                        done();
                        res.send(500, {error: 'db_query_failed'});
                        return;
                    }

                    cb(function (commit_cb) {
                        client.query('COMMIT;', function (err, result) {
                            if (err) {
                                console.log(err);
                                done();
                                res.send(500, {error: 'db_query_failed'});
                                return;
                            }
                            commit_cb();
                        });
                    }, function (rollback_cb) {
                        client.query('ROLLBACK;', function (err, result) {
                            if (err) {
                                console.log(err);
                                done();
                                res.send(500, {error: 'db_query_failed'});
                                return;
                            }
                            rollback_cb();
                        });
                    });
                });
            }
        }

        next();
    });
};

exports.query = function (file, params) {
    return function (req, res, next) {
        if (typeof req.db === 'undefined') {
            console.log(err);
            res.send(500, {error: 'db_not_connected'});
            return;
        }

        fs.readFile(file, 'utf8', function (err, data) {
            if (err) {
                console.log(err);
                res.send(500, {error: 'cannot_open_file'});
                return;
            }

            if (typeof params === 'undefined')
                params = [];

            var p = [];
            for (var i=0; i < params.length; i++) {
                p.push(req.params[params[i]]);
            }

            req.db.query(data, p, function(result) {
                req.db.done();

                return res.json(result.rows);
            });
        });
    };
};
