var exec = require('child_process').exec,
    fs = require('fs');

exports.get_data = function(req, res) {
    if (typeof req.params.project === 'undefined')
        return res.json(400, {error: 'missing_project'});

    if (typeof req.params.type === 'undefined')
        return res.json(400, {error: 'missing_chart_type'});

    /*
     * routes = { 'project': [{ type: 'type', file: 'file', params: 'params}, ...]}
     */
    var routes = {
        'tlbr': [
            /* Availability */
            {
                type: 'avail_current', file: 'queries/tlbr/avail/current.sql'
                , params: [ req.params.region, req.params.state, req.params.id_city ]
            },
            {
                type: 'avail_sub_regions' , file: 'queries/tlbr/avail/sub_regions.sql'
                , params: [ ]
            },
            {
                type: 'avail_sub_states', file: 'queries/tlbr/avail/sub_states.sql'
                , params: [ req.params.region ]
            },
            {
                type: 'avail_sub_cities', file: 'queries/tlbr/avail/sub_cities.sql'
                , params: [ req.params.region, req.params.state ]
            },
            {
                type: 'avail_sub_telecenters', file: 'queries/tlbr/avail/sub_telecenters.sql'
                , params: [ req.params.region, req.params.state, req.params.id_city ]
            },
            {
                type: 'avail_hist', file: 'queries/tlbr/avail/hist.sql'
                , params: [ req.params.region, req.params.state, req.params.id_city ]
            },

            /* Net Usage */
            {
                type: 'net_usage_sub_regions', file: 'queries/tlbr/net_usage/sub_regions.sql'
                , params: [ ]
            },
            {
                type: 'net_usage_sub_states', file: 'queries/tlbr/net_usage/sub_states.sql'
                , params: [ req.params.region ]
            },
            {
                type: 'net_usage_sub_cities', file: 'queries/tlbr/net_usage/sub_cities.sql'
                , params: [ req.params.state ]
            },
            {
                type: 'net_usage_sub_telecenters', file: 'queries/tlbr/net_usage/sub_telecenters.sql'
                , params: [ req.params.id_city ]
            },
            {
                type: 'net_usage_telecenter', file: 'queries/tlbr/net_usage/telecenter.sql'
                , params: [ req.params.id_point ]
            }
        ],

        'gesac': [
            /* Availability */
            {
                type: 'avail_current', file: 'queries/gesac/avail/current.sql'
                , params: [ req.params.region, req.params.state, req.params.id_city ]
            },
            {
                type: 'avail_sub_regions' , file: 'queries/gesac/avail/sub_regions.sql'
                , params: [ ]
            },
            {
                type: 'avail_sub_states', file: 'queries/gesac/avail/sub_states.sql'
                , params: [ req.params.region ]
            },
            {
                type: 'avail_sub_cities', file: 'queries/gesac/avail/sub_cities.sql'
                , params: [ req.params.region, req.params.state ]
            },
            {
                type: 'avail_sub_telecenters', file: 'queries/gesac/avail/sub_telecenters.sql'
                , params: [ req.params.region, req.params.state, req.params.id_city ]
            },
            {
                type: 'avail_hist', file: 'queries/gesac/avail/hist.sql'
                , params: [ req.params.region, req.params.state, req.params.id_city ]
            },

            /* Net Usage */
            {
                type: 'net_usage_sub_regions', file: 'queries/gesac/net_usage/sub_regions.sql'
                , params: [ ]
            },
            {
                type: 'net_usage_sub_states', file: 'queries/gesac/net_usage/sub_states.sql'
                , params: [ req.params.region ]
            },
            {
                type: 'net_usage_sub_cities', file: 'queries/gesac/net_usage/sub_cities.sql'
                , params: [ req.params.state ]
            },
            {
                type: 'net_usage_sub_telecenters', file: 'queries/gesac/net_usage/sub_telecenters.sql'
                , params: [ req.params.id_city ]
            },
            {
                type: 'net_usage_telecenter', file: 'queries/gesac/net_usage/telecenter.sql'
                , params: [ req.params.id_point ]
            }
        ]
    };

    var project = req.params.project;
    if (routes[project] === 'undefined')
        return res.json(400, {error: 'invalid_project'});

    var queries = routes[project];
    for (var i=0; i<queries.length; i++) {
        var query = queries[i];
        if ( query.type === req.params.type ) {
            for (var p=0; p<query.params.length; p++) {
                query.params[p] = query.params[p] || null;
            }
            req.db.queryFromFile(query.file, query.params, function (result) {
                req.db.done();
                res.json(result.rows);
            });

            return;
        }
    }

    res.json(400, {error: 'invalid_params'});
}

exports.get_report = function(req, res) {
    if (typeof req.params.project === 'undefined')
        return res.json(400, {error: 'missing_project'});

    if (typeof req.params.type === 'undefined')
        return res.json(400, {error: 'missing_report_type'});

    /*
     * routes = { 'project': [{ type: 'type', file: 'file', params: 'params}, ...]}
     */
    var routes = {
        'tlbr': [
            /* Availability */
            {
                type: 'avail_report', file: 'telecentroAvail.jrxml'
                , params: [ req.params.id_city ]
            }
        ],

        'gesac': [
            /* Availability */
            {
                type: 'avail_report', file: 'gesacAvail.jrxml'
                , params: [ req.params.id_city ]
            }
        ]
    };

    var project = req.params.project;
    if (routes[project] === 'undefined')
        return res.json(400, {error: 'invalid_project'});

    var queries = routes[project];
    for (var i=0; i<queries.length; i++) {
        var query = queries[i];
        if ( query.type === req.params.type ) {
            for (var p=0; p<query.params.length; p++) {
                query.params[p] = query.params[p] || null;
            }

            var cmdline = 'reports/build-report.sh '+query.file+' '+query.params.join(' ');

            exec(cmdline, function (err, stdout, stderr) {
                if (err) {
                    console.log(err);
                    return res.json(500, {error: 'report_building_failed'});
                }

                res.type('application/pdf');
                res.sendfile(stdout, function (err) {
                    fs.unlink(stdout);
                });
            });

            return;
        }
    }

    res.json(400, {error: 'invalid_params'});
}
